// EMLController.m
//
// MathLink Tutorial: http://www.edenwaith.com/development/tutorials/mathlink/

#import "EMLController.h"
#include <mathlink/mathlink.h>  // also add mathlink.framework to project
                                // The MathLink framework is saved in the 
                                // /Library/Frameworks directory
                                
// #include "mathlink.h"	// This can be used in leiu of the mathlink framework                                
                                // Remember that mathlink.h needs to be copied to
                                // /usr/include/ and libML.a needs to be copied to
                                // /usr/lib/ for this to work properly

@implementation EMLController

// =========================================================================
// (id) init
// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
// Version: 12. October 2003
// Created: 12. October 2003
// =========================================================================
- (id) init
{
    self = [super init];
    
    formula = [[NSMutableString alloc] init];
    
    return self;
}

// =========================================================================
// (void) dealloc
// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
// Version: 12. October 2003
// Created: 12. October 2003
// =========================================================================
- (void) dealloc 
{
    [formula dealloc];
    [super dealloc];
}

// =========================================================================
// (BOOL) applicationShould....:(NSApplication *)theApplication
// -------------------------------------------------------------------------
// Terminate the program when the last window closes
// Need to connect File Owner to EMLController as a delegate for this to
// work correctly
// -------------------------------------------------------------------------
// Version: 30. November 2003 21:14
// Created: 30. November 2003 21:14
// =========================================================================
- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)theApplication 
{
    return YES;
}

// =========================================================================
// (IBAction) getInput : (id) sender
// -------------------------------------------------------------------------
// This uses an NSMatrix to get the title in the selected cell of the matrix.
// This greatly simplifies the process of reading in the pressed key verses
// the original implementation in EdenMath.
// -------------------------------------------------------------------------
// Version: 12. October 2003
// Created: 12. October 2003
// =========================================================================
- (IBAction) getInput : (id) sender
{
    id theCell = [sender selectedCell];
    NSString *theTitle = [theCell title];
    
    [formula appendString: theTitle];
    
    [display setStringValue: formula];
}

// =========================================================================
// (IBAction) clearDisplay : (id) sender
// -------------------------------------------------------------------------
// The name says it all.  This clears the display.
// -------------------------------------------------------------------------
// Version: 12. October 2003
// Created: 12. October 2003
// =========================================================================
- (IBAction) clearDisplay : (id) sender
{
    [formula setString: @""];
    [display setStringValue: @""];
}

// =========================================================================
// (IBAction) solve : (id) sender
// -------------------------------------------------------------------------
// This contains the main brunt of the MathLink code.  If you are using a
// different version of Mathematica, change the second argument in argv[]
// to what is appropriate (i.e. .../Mathematica 5.0.app/Contents/...).
// -------------------------------------------------------------------------
// Version: 12. October 2003
// Created: 12. October 2003
// =========================================================================
- (IBAction) solve : (id) sender
{
    float sum = 0;
    int argc = 4;
    char *argv[5] = {"-linkname",
                     " '/Applications/Mathematica 4.2.app/Contents/MacOS/MathKernel' -mathlink",
                     "-linkmode",
                     "launch",
                     NULL};
    MLINK lp;
    MLEnvironment env;
    char *expr;
        
    expr = [[display stringValue] cString];
    
    env = MLInitialize(NULL);
    
    if (env == NULL)
    {
        NSBeep();
        NSRunAlertPanel(@"Error", @"MathLink Environment could not be initialized.", @"OK", nil, nil);
        [NSApp terminate:self];
    }
    
    lp = MLOpen(argc, argv);
    
    if (lp == NULL)
    {
        NSBeep();
        NSRunAlertPanel(@"Error", @"MathLink could not be opened.", @"OK", nil, nil);
        [NSApp terminate:self];
    }

    // Send the expression from the display to the Mathematica kernel to have
    // it evaluate the expression.
    MLPutFunction(lp, "EvaluatePacket", 1);
    MLPutFunction(lp, "ToExpression", 1);
    MLPutString(lp, expr);
    MLEndPacket(lp);

    while (MLNextPacket(lp) != RETURNPKT) MLNewPacket(lp);

    MLGetFloat(lp, &sum);

    MLClose(lp);
    MLDeinitialize(env);
                                                       
    [display setFloatValue: sum];
    
    [formula setString: @""]; // reinitialize the formula string
}

@end
